#include "id_enclave_u.h"
#include <errno.h>

typedef struct ms_ide_get_id_t {
	sgx_status_t ms_retval;
	sgx_key_128bit_t* ms_p_id;
} ms_ide_get_id_t;

typedef struct ms_ide_get_pce_encrypt_key_t {
	sgx_status_t ms_retval;
	const sgx_target_info_t* ms_pce_target_info;
	sgx_report_t* ms_p_ide_report;
	uint8_t ms_crypto_suite;
	uint16_t ms_cert_key_type;
	uint32_t ms_key_size;
	uint8_t* ms_p_public_key;
} ms_ide_get_pce_encrypt_key_t;

static const struct {
	size_t nr_ocall;
	void * table[1];
} ocall_table_id_enclave = {
	0,
	{ NULL },
};
sgx_status_t ide_get_id(sgx_enclave_id_t eid, sgx_status_t* retval, sgx_key_128bit_t* p_id)
{
	sgx_status_t status;
	ms_ide_get_id_t ms;
	ms.ms_p_id = p_id;
	status = sgx_ecall(eid, 0, &ocall_table_id_enclave, &ms);
	if (status == SGX_SUCCESS && retval) *retval = ms.ms_retval;
	return status;
}

sgx_status_t ide_get_pce_encrypt_key(sgx_enclave_id_t eid, sgx_status_t* retval, const sgx_target_info_t* pce_target_info, sgx_report_t* p_ide_report, uint8_t crypto_suite, uint16_t cert_key_type, uint32_t key_size, uint8_t* p_public_key)
{
	sgx_status_t status;
	ms_ide_get_pce_encrypt_key_t ms;
	ms.ms_pce_target_info = pce_target_info;
	ms.ms_p_ide_report = p_ide_report;
	ms.ms_crypto_suite = crypto_suite;
	ms.ms_cert_key_type = cert_key_type;
	ms.ms_key_size = key_size;
	ms.ms_p_public_key = p_public_key;
	status = sgx_ecall(eid, 1, &ocall_table_id_enclave, &ms);
	if (status == SGX_SUCCESS && retval) *retval = ms.ms_retval;
	return status;
}

