/*
 * Copyright (C) 2011-2021 Intel Corporation. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of Intel Corporation nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
/**
 * File: FSUefi.h 
 *   
 * Description: Definition for the class to send communicate with BIOS 
 * via UEFI variables. Implements the IUefi interface class. 
 */
#ifndef __FS_UEFI_H
#define __FS_UEFI_H

#include <string>
#include "IUefi.h"
#include "MultiPackageDefs.h"

using std::string;

class FSUefi : public IUefi {
    public:
        FSUefi(string uefiPath, const LogLevel logLevel) : m_uefiAbsPath(uefiPath), m_logLevel(logLevel) {};
        virtual uint8_t* readUEFIVar(const char* varName, size_t &dataSize);
        virtual int writeUEFIVar(const char* varName, const uint8_t* data, size_t dataSize, bool create);
    private:
        static const string createFullPath(const string &path, const char* uefiVarName);
        static long fdGetVarFileSize(int fd);
        string m_uefiAbsPath;
        LogLevel m_logLevel;
};

#endif  // #ifndef __FS_UEFI_H

